import { memo, useRef, useState, useMemo, useEffect } from 'react';
import { Handle, Position } from 'reactflow';
import { RelationNodeAttributes, SchemaReactflowRelationNode, toHandleId } from '../../../model';
import styles from './relationpill.module.scss';
import {
  setQuerybuilderNodes,
  useAppDispatch,
  useConfig,
  useQuerybuilderGraph,
  useQuerybuilderSettings,
} from '@graphpolaris/shared/lib/data-access';
import { addWarning } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import { setQuerybuilderGraphology, toQuerybuilderGraphology } from '@graphpolaris/shared/lib/data-access/store/querybuilderSlice';
import graphology from 'graphology';

/**
 * Component to render a relation flow element
 * @param { FlowElement<RelationData>} param0 The data of a relation flow element.
 */
export const RelationPill = memo((node: SchemaReactflowRelationNode) => {
  const data = node.data;
  const graph = useQuerybuilderGraph();
  const graphologyGraph = useMemo(() => toQuerybuilderGraphology(graph), [graph]);
  const settings = useQuerybuilderSettings();
  const dispatch = useAppDispatch();
  const graphologyNodeAttributes = useMemo<RelationNodeAttributes | undefined>(
    () => (graphologyGraph.hasNode(node.id) ? { ...(graphologyGraph.getNodeAttributes(node.id) as RelationNodeAttributes) } : undefined),
    [node.id]
  );
  const [depth, setDepth] = useState<{ min: number; max: number }>({
    min: data.depth.min || settings.depth.min,
    max: data.depth.max || settings.depth.max,
  });

  useEffect(() => {
    setDepth({ min: data.depth.min || settings.depth.min, max: data.depth.max || settings.depth.max });
  }, [data.depth]);

  const onNodeUpdated = () => {
    if (depth.min < 0) {
      dispatch(addWarning('The minimum depth cannot be smaller than 0'));
    } else if (depth.max > 99) {
      dispatch(addWarning('The maximum depth cannot be larger than 99'));
    } else if (depth.min >= depth.max) {
      dispatch(addWarning('The minimum depth cannot be larger than the maximum depth'));
    } else {
      graphologyGraph.setNodeAttribute<any>(node.id, 'depth', depth);
      dispatch(setQuerybuilderGraphology(graphologyGraph));
    }
  };

  const calcWidth = (data: number) => {
    return data.toString().length + 0.5 + 'ch';
  };

  return (
    <div className={styles.relation}>
      <div className={styles.relationWrapper}>
        <span className={styles.relationHandleFiller}>
          {data.leftEntityHandleId && (
            <Handle id={toHandleId(data.leftEntityHandleId)} type="target" position={Position.Left} className={styles.relationHandleLeft} />
          )}
        </span>
        {/* <span className={styles.relationHandleFiller}>
          <Handle
            id={getHandleId(data.name, data.type, Handles.ToAttribute, '')}
            type="target"
            position={Position.Left}
            className={styles.relationHandleAttribute + ' ' + (false ? styles.handleConnectedFill : '')}
          />
        </span>
        <span className={styles.relationHandleFiller}>
          <Handle
            id={getHandleId(data.name, data.type, Handles.ReceiveFunction, '')}
            type="target"
            position={Position.Left}
            className={styles.relationHandleFunction + ' ' + (false ? styles.handleConnectedFill : '')}
          />
        </span> */}
        <div className={styles.relationDataWrapper}>
          <span className={styles.relationSpan}>{data?.name}</span>
          <span className={styles.relationInputHolder}>
            <span>[</span>
            <input
              className={
                'bg-inherit text-center appearance-none mx-0.5 rounded-sm ' +
                (depth.min < 0 || depth.min >= depth.max ? ' bg-red-400 ' : '')
              }
              style={{ maxWidth: calcWidth(depth.min) }}
              type="number"
              min={0}
              placeholder={'?'}
              value={depth.min}
              onChange={(e) => {
                setDepth({ ...depth, min: parseInt(e.target.value) });
              }}
              onBlur={(e) => {
                onNodeUpdated();
              }}
              onKeyDown={(e) => {
                if (e.key === 'Enter') {
                  onNodeUpdated();
                }
              }}
            ></input>
            <span>..</span>
            <input
              className={
                'bg-inherit text-center appearance-none mx-0.5 rounded-sm ' +
                (depth.max > 99 || depth.min >= depth.max ? ' bg-red-400 ' : '')
              }
              style={{ maxWidth: calcWidth(depth.max) }}
              type="number"
              min={1}
              placeholder={'?'}
              value={depth.max}
              onChange={(e) => {
                setDepth({ ...depth, max: parseInt(e.target.value) });
              }}
              onBlur={(e) => {
                onNodeUpdated();
              }}
              onKeyDown={(e) => {
                if (e.key === 'Enter') {
                  onNodeUpdated();
                }
              }}
            ></input>
            <span>]</span>
          </span>
        </div>
        <span className={styles.relationHandleFiller}>
          {data.rightEntityHandleId && (
            <Handle
              id={toHandleId(data.rightEntityHandleId)}
              type="source"
              position={Position.Right}
              className={styles.relationHandleRight + ' ' + (false ? styles.handleConnectedBorderRight : '')}
            />
          )}
        </span>
      </div>
    </div>
  );
});

RelationPill.displayName = 'RelationPill';
export default RelationPill;
